# GNU Enterprise Application Server - Class Repository: Class
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Class.py,v 1.10 2003/10/21 12:26:50 jvetter Exp $

import types
from Base import *
from Namespace import *
from Property import *

# =============================================================================
# Dictionary with classes
# =============================================================================
class ClassDict (BaseDictionary):

  # ---------------------------------------------------------------------------
  # Create a dictionary with all classes [from a specified module]
  # ---------------------------------------------------------------------------
  def __init__ (self, session, moduleDict, module = None, definition = None):
    BaseDictionary.__init__ (self, session)

    self.modules    = moduleDict
    self._module    = module

    self.definition = definition

    if not definition:
      self.definition = self.modules.definition

    # if we have an initial definition, use all it's classes
    if self.definition:
      list = [klass [1] for klass in self.definition.classes ()]
      self._updateCache (list)

    # if a module is specified, fetch all it's classes 
    if module is not None:
      if self.modules.classdict:
        for item in self.modules.classdict.values ():
          if item.module.fullName == module.fullName:
            self._cache [item.fullName] = item
      else:
        self._updateCache (self._requestData (module.gnue_id))

    # otherwise preload the cache with all available classes !
    else:
      self._updateCache (self._requestData ())

  # ---------------------------------------------------------------------------
  # Update the cache with all business objects in list
  # ---------------------------------------------------------------------------
  def _updateCache (self, list):
    for item in list:
      klass = self._class_from_cache (item)
      self._cache [klass.fullName] = klass

  # ---------------------------------------------------------------------------
  # Request a list of matching business objects
  # ---------------------------------------------------------------------------
  def _requestData (self, moduleId = None, className = None):
    if not self.session:
      return []

    cond = []
    if moduleId:
      cond = [["eq", ""], ["field", "gnue_module"], ["const", moduleId]]

    if className:
      cond += [["eq", ""], ["field", "gnue_name"], ["const", className]]

    return self.session.find ("gnue_class", cond, 
                              ["gnue_module", "gnue_name"],
                              ["gnue_module", "gnue_name"])

  # ---------------------------------------------------------------------------
  # Return an item from the cache using the ModuleDict for module verification
  # ---------------------------------------------------------------------------
  def _class_from_cache (self, item):
    module = self._module_from_cache (item)
    return Class (self, module, item)

  # ---------------------------------------------------------------------------
  # Return the module of a cached item
  # ---------------------------------------------------------------------------
  def _module_from_cache (self, item):
    for module in self.modules.values ():
      if module.gnue_id == item.gnue_module:
	return module
    raise EBadReference (item.gnue_name, "gnue_module")

  # ---------------------------------------------------------------------------
  # Return a class object if it exists
  # ---------------------------------------------------------------------------
  def get (self, name):
    (moduleName, className) = splitName (name)
    module = self.modules [moduleName]

    # is the class already cached ?
    if self._cache.has_key (name):
      return self._cache [name]

    # since we had no match cached, ask for it
    res = self._requestData (module.gnue_id, className)

    if len (res) == 1:
      self._updateCache (res)
      return self._cache [name]

    raise EClassNotFound (name)



# =============================================================================
# An implementation of a single class
# =============================================================================
class Class:
  
  # ---------------------------------------------------------------------------
  # Wrapping a business object class
  # ---------------------------------------------------------------------------
  def __init__ (self, classDict, module, object):
    self._object    = object
    self.classes    = classDict
    self.module     = module
    self.definition = classDict.definition
    self.fullName   = createName (module.gnue_name, self.gnue_name)
    self.table      = self.fullName

    self.properties = PropertyDict (classDict.session, classDict.modules, self)


  # ---------------------------------------------------------------------------
  # Attribute access is redirected into the gnue-namespace
  # ---------------------------------------------------------------------------
  def __getattr__ (self, attr):
    value = getattr (self._object, attr)
    if isinstance (value, types.UnicodeType):
      return value.encode ('utf-8')
    else:
      return value

  # ---------------------------------------------------------------------------
  # Return a property by name
  # ---------------------------------------------------------------------------
  def findProp (self, name):
    return self.properties [name]
