# GNU Enterprise Application Server - Class Repository: Namespace
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Namespace.py,v 1.4 2003/10/19 22:08:04 reinhard Exp $

import string

# =============================================================================
# Base class for all class-repository exceptions
# =============================================================================
class EClassRepository (Exception):
  def __init__ (self, text):
    self._msg = text
    Exception.__init__ (self, self._msg)

  def __repr__ (self):
    return self._msg

# =============================================================================
# Malformed names
# =============================================================================
class EInvalidName (EClassRepository):
  def __init__ (self, name):
    msg = "'%s' is not a valid, fully qualified identifier." % name
    EClassRepository.__init__ (self, msg)


# =============================================================================
# Object references a non-existing id
# =============================================================================
class EBadReference (EClassRepository):
  def __init__ (self, item, reference):
    msg = "Reference %s of object %s is invalid." % (reference, item)
    EClassRepository.__init__ (self, msg)

# =============================================================================
# Requested module does not exist
# =============================================================================
class EModuleNotFound (EClassRepository):
  def __init__ (self, module):
    msg = "Module '%s' not found in class repository." % module
    EClassRepository.__init__ (self, msg)

# =============================================================================
# Requested class does not exist
# =============================================================================
class EClassNotFound (EClassRepository):
  def __init__ (self, klass):
    msg = "Class '%s' not found in class repository." % klass
    EClassRepository.__init__ (self, msg)

# =============================================================================
# Requested property does not exist
# =============================================================================
class EPropertyNotFound (EClassRepository):
  def __init__ (self, classname, property):
    msg = "Class '%s' has no property '%s'." % (classname, property)
    EClassRepository.__init__ (self, msg)

# =============================================================================
# Class repository definition file has an invalid structure
# =============================================================================
class EInvalidDefinition (EClassRepository):
  def __init__ (self, text):
    msg = "Invalid repository definition: %s" % text
    EClassRepository.__init__ (self, msg)

# =============================================================================
# Namespace constants
# =============================================================================

NSID_INVALID  = -1
NSID_MODULE   =  1
NSID_CLASS    =  2
NSID_PROPERTY =  3

# -----------------------------------------------------------------------------
# Split a fully qualified name into a tuple (namespace, identifier)
# -----------------------------------------------------------------------------
def splitName (name):
  parts = string.split (name, '_')

  if len (parts) == 1:
    return ('', name)

  elif len (parts) == 2:
    return (parts [0], parts [1])

  raise EInvalidName (name)

# -----------------------------------------------------------------------------
# Create a fully qualified name from a namespace and an identifier
# -----------------------------------------------------------------------------
def createName (namespace, identifier):
  return "%s_%s" % (namespace, identifier)

# -----------------------------------------------------------------------------
# Try to figure out which NSID_* constant applies to identifier
# TODO: implement some methods to recognize class-/typeinformation about
#       the classrepository itself.
# -----------------------------------------------------------------------------
def getNamespaceId (identifier):
  # a valid identifier must contain at least one character
  if not identifier:
    return NSID_INVALID

  # a module-identifier must not contain '.'- or '_'-characters; i.e. 'gnue'
  if not 1 in [char in '._' for char in identifier]:
    return NSID_MODULE

  # class-identifiers are formed out of two module-identifieres separated by an
  # underscore, i.e. 'gnue_module'
  parts = string.split (identifier, '_')
  if len (parts) == 2:
    if getNamespaceId (parts [0]) == NSID_MODULE and \
       getNamespaceId (parts [1]) == NSID_MODULE:
      return NSID_CLASS

  # property-identifiers are fromed out of two class-identifiers separated by a
  # single dot, i.e. 'gnue_module.gnue_id'
  parts = string.split (identifier, '.')
  if len (parts) == 2:
    if getNamespaceId (parts [0]) == NSID_CLASS and \
       getNamespaceId (parts [1]) == NSID_CLASS:
      return NSID_PROPERTY

  # since no pattern matched until here, we must assume the identifier as
  # invalid.
  return NSID_INVALID
