# GNU Enterprise Application Server - Class Repository: Property
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Property.py,v 1.9 2003/10/19 22:08:04 reinhard Exp $

import types
from Base import *
from Namespace import *

# =============================================================================
# Dictionary with all properties of a given class
# =============================================================================
class PropertyDict (BaseDictionary):

  # ---------------------------------------------------------------------------
  # Construct a Property-Dictionary for class klass
  # ---------------------------------------------------------------------------
  def __init__ (self, session, moduleDict, klass, definition = None):
    BaseDictionary.__init__ (self, session)

    self.modules    = moduleDict
    self.klass      = klass
    self.definition = definition

    if not self.definition:
      self.definition = klass.definition

    if not self.definition:
      self.definition = moduleDict.definition

    if self.definition:
      list = [prop [1] for prop in self.definition.properties (klass.fullName)]
      self._updateCache (list)

    self._updateCache (self._requestData (self.klass.gnue_id))

  # ---------------------------------------------------------------------------
  # Update cache with all business objects in list
  # ---------------------------------------------------------------------------
  def _updateCache (self, list):
    for item in list:
      property = self._property_from_cache (item)
      self._cache [property.fullName] = property

  # ---------------------------------------------------------------------------
  # Read a list of properties from the datasource
  # ---------------------------------------------------------------------------
  def _requestData (self, classId, propertyName = None):
    # Special case: if there is already a property with a matching class id
    # we assume all properties of the requesed class are already cached
    for prop in self.values ():
      if prop.gnue_class == classId: 
	return []

    cond = [["eq", ""], ["field", "gnue_class"], ["const", classId]]

    if propertyName:
      cond += [["eq", ""], ["field", "gnue_name"], ["const", propertyName]]

    return self.session.find ("gnue_property", cond, ["gnue_name"], 
                               ["gnue_class", "gnue_module", "gnue_name", 
	 		        "gnue_type", "gnue_length", "gnue_scale",
                                "gnue_comment"])

  # ---------------------------------------------------------------------------
  # Get a property from the cache
  # ---------------------------------------------------------------------------
  def _property_from_cache (self, item):
    return Property (self._module_from_cache (item), self.klass, item)

  # ---------------------------------------------------------------------------
  # Get a module from a cache-item
  # ---------------------------------------------------------------------------
  def _module_from_cache (self, item):
    for module in self.modules.values ():
      if module.gnue_id == item.gnue_module:
	return module
    raise EBadReference (item.gnue_name, "gnue_module")


  # ---------------------------------------------------------------------------
  # Return a Property object if it exists
  # ---------------------------------------------------------------------------
  def get (self, name):
    (moduleName, propName) = splitName (name)
    module = self.modules [moduleName]

    # is the property already cached
    if self.has_key (name):
      return self._cache [name]

    # since we had no match in the cache ask for it
    res = self._requestData (self.klass.gnue_id, name)

    if len (res) == 1:
      self._updateCache (res)
      return self._cache [name]

    raise EPropertyNotFound (self.klass.fullName, name)


# =============================================================================
# A Business Object Property
# =============================================================================
class Property:

  # ---------------------------------------------------------------------------
  # Construct a new property from module, class and business object
  # ---------------------------------------------------------------------------
  def __init__ (self, module, klass, object):
    self.module   = module
    self.klass    = klass
    self._object  = object

    # gnue_property.gnue_name is already a fully qualified identifier
    self.fullName = createName (module.gnue_name, self.gnue_name)
    self.column   = self.fullName
    if self.gnue_type == 'string':
      self.fullType = '%s(%d)' % (self.gnue_type, int (self.gnue_length))
    elif self.gnue_type == 'number':
      if self.gnue_scale == 0:
        self.fullType = '%s(%d)' % (self.gnue_type, int (self.gnue_length))
      else:
        self.fullType = '%s(%d.%d)' % (self.gnue_type, int (self.gnue_length),
                                       int (self.gnue_scale))
    else:
      self.fullType = self.gnue_type

  # ---------------------------------------------------------------------------
  # all unknown attributes are routed to the gnue-namespace
  # ---------------------------------------------------------------------------
  def __getattr__ (self, attr):
    value = getattr (self._object, attr)
    if isinstance (value, types.UnicodeType):
      return value.encode ('utf-8')
    else:
      return value
