# GNU Enterprise Application Server - GNUe Schema Support
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: SchemaSupport.py,v 1.3 2003/10/21 12:26:50 jvetter Exp $

from gnue.appserver.classrep import Namespace

# read schema support
from gnue.common.schema import GSParser
from gnue.common.utils.FileUtils import openResource, dyn_import

# write schema support
from gnue.common.schema.Objects import *
from gnue.common.definitions.GParserHelpers import GContent

SCHEMA_STORE_BOTH = 0
SCHEMA_STORE_SCHEMA = 1
SCHEMA_STORE_DATA = 2
SCHEMA_TYPES = { 'string'  : 'string', 
                 'number'  : 'number',
                 'boolean' : 'boolean',
                 'date'    : 'date',
                 'time'    : 'time',
                 'datetime': 'timestamp' }


class SchemaSupport:

  def __init__(self, modules, classes):
    self._modules = modules
    self._classes = classes

  # ---------------------------------------------------------------------------
  # Load a Schema from a file and store it in the class repository
  # Parameter: filename:  name of file to load
  #            overwrite: if true, old class definitions will be removed
  #            install:   if true, database tables for the class will be created
  # TODO: a) make this code transactional, i.e. first check, if everything can 
  #          be done without breaks, then execute.
  #       b) add global class repository locking
  # ---------------------------------------------------------------------------
  def loadSchemaFromFile (self, filename, overwrite=0, install=0):
    input = openResource (filename)
    schema = GSParser.loadFile (input)

    for table in schema.findChildrenOfType ('GSTable', allowAllChildren = 1):
      print 'adding Table %s' % table.name

      for field in table.findChildrenOfType ('GSField', allowAllChildren = 1):
        print '   with field %s:' % field.name

    if overwrite:
      raise 'Overwrite support not working yet.'
      
    if install:
      # TODO: this job should be done in a quite general way,
      #       to let the code be reused in reaction on  manual class
      #       repository changes.
      raise 'Creation of Schema tables is not supported yet'

  # ---------------------------------------------------------------------------
  # Write a Schema to a file
  # Parameter: filename:  name of file to write to
  #            classlist: list of classes to write to
  # TODO: a) add global class repository locking
  #       b) add a (include system tables) parameter
  # ---------------------------------------------------------------------------
  def writeSchemaToFile (self, filename, classlist = None,
                         type = SCHEMA_STORE_BOTH):
    schema = GSSchema()
    schema.title   = 'Appserver Schema Dump'
    schema.author  = 'Appserver SchemaSupport'
    schema.version = '1.0'
    # schema.description=''
      
    gstables = GSTables (schema)

    # use either the given classlist or a list of all classes available
    tablelist = classlist or self._classes.keys ()

    # create the schema definition
    if (type == SCHEMA_STORE_BOTH) or (type == SCHEMA_STORE_SCHEMA):
      for tablename in tablelist:
        self.createTableDef (gstables, tablename)

    # optionally create the datalist
    if (type == SCHEMA_STORE_BOTH) or (type == SCHEMA_STORE_DATA):
      gsdata = GSData (schema)
      self.createDataList (gsdata, tablelist)
                
    # create the .gsd-file
    dest = open (filename, 'w')
      
    dest.write ("""<?xml version="1.0"?>\n""" + 
      """<!-- Schema definition created by GNUe Appserver's Schema Support\n"""+
      """Dropped Schema includes %s  -->\n""" % tablelist)

    dest.write (schema.dumpXML ())
    dest.close ()

  # ---------------------------------------------------------------------------
  # Create the tabledefinition for tablename
  # ---------------------------------------------------------------------------
  def createTableDef (self, gstables, tablename):
    table = GSTable (gstables)
    table.name = tablename
    #table.description = ''
      
    # add fields listing (required)
    fields = GSFields (table)
    GSConstraints (table)
    GSIndexes (table)
      
    for property in self._classes [tablename].properties.values ():
      # add one field per property
      field = GSField (fields)
      field.name        = property.fullName
      field.description = property.gnue_comment

      # use valid types as is
      if property.gnue_type in SCHEMA_TYPES.keys ():
        field.type        = SCHEMA_TYPES [property.gnue_type]
        field.length      = int (property.gnue_length)
        field.precision   = int (property.gnue_scale)
      # everything else should be a reference
      else:
        field.type        = 'string'
        field.length      = 32
        field.precision   = 0

      # TODO: where should 'gnue_module' be stored?

      # set gnue_id to be a primary key
      if (property.fullName == 'gnue_id') and (property.gnue_type == 'id'):
        pk = GSPrimaryKey (table)
        pk.name = 'gnue_id_pk_' + property.klass.fullName

        pkf = GSPKField(pk)
        pkf.name = 'gnue_id'
          
                  
  # ---------------------------------------------------------------------------
  # Dump all tabledata from a given tablelist
  # ---------------------------------------------------------------------------
  def createDataList(self, gsdata, tablelist):
    # get a unique list of all modules enlisted in tablelist
    modlist = {}
    for table in tablelist:
      (module, classname) = Namespace.splitName (table)
      modlist [module] = 1

    # dump all modules enlisted in tablelist
    moddata = GSTableData (gsdata)
    moddata.name      = 'gnue_module_dump'
    moddata.tablename = 'gnue_module'
    modrows = GSRows (moddata)

    for module in modlist.keys ():
      moddef = self._modules [module]

      row = GSRow (modrows)
      # we use the same sort order here as BaseDictionary.values ()
      self._buildValue (row, 'gnue_comment', moddef.gnue_comment)
      self._buildValue (row, 'gnue_id'     , moddef.gnue_id)
      self._buildValue (row, 'gnue_name'   , moddef.gnue_name)

    # dump all classes enlisted in tablelist
    tabledata = GSTableData (gsdata)
    tabledata.name      = 'gnue_class_dump'
    tabledata.tablename = 'gnue_class'
    tablerows = GSRows (tabledata)
      
    # and all their properties 
    propdata = GSTableData (gsdata)
    propdata.name      = 'gnue_property_dump'
    propdata.tablename = 'gnue_property'
    proprows = GSRows (propdata)

    for tablename in tablelist:
      classdef = self._classes [tablename]
          
      # save tabledata
      row = GSRow (tablerows)              
      self._buildValue (row, 'gnue_comment', classdef.gnue_comment)
      self._buildValue (row, 'gnue_id'     , classdef.gnue_id)
      self._buildValue (row, 'gnue_module' , classdef.gnue_module)
      self._buildValue (row, 'gnue_name'   , classdef.gnue_name)
      
      for property in classdef.properties.values ():
        # add one field per property
        row = GSRow (proprows)                           
        self._buildValue (row, 'gnue_class' ,  property.gnue_class)
        self._buildValue (row, 'gnue_comment', property.gnue_comment)
        self._buildValue (row, 'gnue_id'    ,  property.gnue_id)
        self._buildValue (row, 'gnue_length',  int (property.gnue_length))
        self._buildValue (row, 'gnue_module',  property.gnue_module)
        self._buildValue (row, 'gnue_name'  ,  property.gnue_name)
        self._buildValue (row, 'gnue_scale' ,  int (property.gnue_scale))
        self._buildValue (row, 'gnue_type'  ,  property.gnue_type)
                  

  # ---------------------------------------------------------------------------
  # Create a new field in a row and populate it with a value
  # ---------------------------------------------------------------------------
  def _buildValue(self, row, field, data):
    val = GSValue (row)
    GContent (val, "'%s'" % data)

# =============================================================================
# test program
# =============================================================================

if __name__ == '__main__':
  from gnue.appserver.test import testApp
  
  app=testApp()
  
  sm = app.getSessionManager ()
  
  gsdSupp = SchemaSupport (sm.modules, sm.classes)

  gsdSupp.writeSchemaToFile ('AS_schema.gsd')
  gsdSupp.writeSchemaToFile ('AS_schema2.gsd', ['address_person'])

 #  gsdSupp.loadSchemaFromFile('AS_schema.gsd',overwrite=0)
  
