# GNU Enterprise Application Server - Instance Object
#
# Copyright 2001-2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: geasInstance.py,v 1.32 2003/10/19 21:46:17 reinhard Exp $

import types
import mx.DateTime
import mx.DateTime.ISO

# =============================================================================
# Instance class
# =============================================================================

class geasInstance:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self, list, record, classdef):
    self._list = list
    self._record = record
    self._classdef = classdef

  # ---------------------------------------------------------------------------
  # Get the value of a property and convert it into RPC-transportable datatype
  # ---------------------------------------------------------------------------

  def _getValue (self, propertyname):
    propertydef = self._classdef.properties [propertyname]
    value = self._record.getField (propertydef.column)

    # Always return an empty string for NULL
    if value is None:
      return ""

    # String property (the id is actually a string, too)
    if propertydef.gnue_type == "id" or \
       propertydef.gnue_type == "string":      
      if isinstance (value, types.UnicodeType):
        # encode unicode values to utf-8 (normal case)
        return value.encode ('utf-8')
      elif isinstance (value, types.StringType):
        # if not UnicodeType then return normal string
        return value
      else:
        raise Exception, ("Database returned invalid value '%s' for " + \
                          "property '%s'") % (repr (value), propertyname)
    

    # Number property
    elif propertydef.gnue_type == "number":
      try:
        if propertydef.gnue_scale == 0:
          # ... without fractional part
          return int (value)
        else:
          # ... with fractional part
          return float (value)
      except ValueError:
        raise Exception, ("Database returned invalid value '%s' for " + \
                            "property '%s'") % (repr (value), propertyname)
      
    # Date property
    elif propertydef.gnue_type == "date":
      if isinstance (value, mx.DateTime.DateTimeType):
        return value.date
      elif isinstance (value, types.UnicodeType):
        # TODO: Check if valid date
        return value.encode ('utf-8')
      elif isinstance (value, types.StringType):
        # TODO: Check if valid date
        return value
      else:
        raise Exception, ("Database returned invalid value '%s' for " + \
                          "property '%s'") % (repr (value), propertyname)

    # Time property
    elif propertydef.gnue_type == "time":
      if isinstance (value, mx.DateTime.DateTimeType):
        return value.time
      # some dbdrivers return DateTimeDelta values
      elif isinstance (value, mx.DateTime.DateTimeDeltaType):
        return str (value)
      elif isinstance (value, types.UnicodeType):
        # TODO: Check if valid time
        return value.encode ('utf-8')
      elif isinstance (value, types.StringType):
        # TODO: Check if valid time
        return value
      else:
        raise Exception, ("Database returned invalid value '%s' for " + \
                          "property '%s'") % (repr (value), propertyname)

    # Date/Time property
    elif propertydef.gnue_type == "datetime":
      if isinstance (value, mx.DateTime.DateTimeType):
        return value.date + " " + value.time
      elif isinstance (value, types.UnicodeType):
        # TODO: Check if valid datetime
        return value.encode ('utf-8')
      elif isinstance (value, types.StringType):
        # TODO: Check if valid datetime
        return value
      else:
        raise Exception, ("Database returned invalid value '%s' for " + \
                          "property '%s'") % (repr (value), propertyname)

    # Boolean property
    elif propertydef.gnue_type == "boolean":
      if value in [0, "0", "f", "F", "false", "FALSE", "n", "N", "no", "NO"]:
        return 0
      elif value in [1, "1", "t", "T", "true", "true", "y", "Y", "yes", "YES"]:
        return 1
      else:
        raise Exception, ("Database returned invalid value '%s' for " + \
                          "property '%s'") % (repr (value), propertyname)

    # Reference property: gnue_type is a classname
    elif self._classdef.classes.has_key (propertydef.gnue_type):
      if isinstance (value, types.UnicodeType):
        # encode unicode values to utf-8 (normal case)
        return value.encode ('utf-8')
      elif isinstance (value, types.StringType):
        # if not UnicodeType then return normal string
        return value
      else:
        raise Exception, ("Database returned invalid value '%s' for " + \
                          "property '%s'") % (repr (value), propertyname)

    # TODO: Missing property types:
    #       * list properties
    #       * calculated properties (must have triggers working first)

    else:
      raise Exception, "Property type '%s' not defined" % propertydef.gnue_type

  # ---------------------------------------------------------------------------
  # Get the values of a list of properties
  # ---------------------------------------------------------------------------

  def get (self, propertylist):
    result = []
    for property in propertylist:
      result.append (self._getValue (property))
    return result

  # ---------------------------------------------------------------------------
  # Set the value of a property and convert it into native datatype
  # ---------------------------------------------------------------------------

  def _putValue (self, propertyname, value):
    propertydef = self._classdef.properties [propertyname]

    # TODO: add more tests to see if the new value is valid, wrt max length

    # NULL
    if value == "":
      value = None

    # String property (the id is actually a string, too)
    elif propertydef.gnue_type == "id" or \
         propertydef.gnue_type == "string":      
      if not isinstance (value, types.StringType):
        value = str (value)
      # decode unicode values from utf-8 
      value = unicode (value, 'utf-8')

    # Number property
    elif propertydef.gnue_type == "number":
      try:
        if propertydef.gnue_scale == 0:
          # ... without fractional part
          value = int (value)
        else:
          # ... with fractional part
          value = float (value)
      except ValueError:
        raise Exception, "Invalid value '%s' for property '%s'" % \
                         (value, propertyname)

    # Date property
    elif propertydef.gnue_type == "date":
      try:
        # Workaround: gnue-common's dbdriver for appserver post a datetime
        # value for date properties. So silently accept that.
        value = mx.DateTime.ISO.ParseDateTime (value)
      except ValueError:
        raise Exception, "Invalid value '%s' for property '%s'" % \
                         (value, propertyname)

    # Time property
    elif propertydef.gnue_type == "time":
      try:
        # Workaround: gnue-common's dbdriver for appserver post a datetime
        # value for time properties. So silently accept that.
        value = mx.DateTime.ISO.ParseDateTime (value)
      except ValueError:
        raise Exception, "Invalid value '%s' for property '%s'" % \
                         (value, propertyname)

    # Date/Time property
    elif propertydef.gnue_type == "datetime":
      try:
        value = mx.DateTime.ISO.ParseDateTime (value)
      except ValueError:
        raise Exception, "Invalid value '%s' for property '%s'" % \
                         (value, propertyname)

    # Boolean property
    elif propertydef.gnue_type == "boolean":
      if value in [0, "0", "f", "F", "false", "FALSE", "n", "N", "no", "NO"]:
        value = 0
      elif value in [1, "1", "t", "T", "true", "true", "y", "Y", "yes", "YES"]:
        value = 1
      else:
        raise Exception, "Invalid value '%s' for property '%s'" % \
                         (value, propertyname)

    self._record.setField (propertydef.column, value)

  # ---------------------------------------------------------------------------
  # Set the values of a list of properties
  # ---------------------------------------------------------------------------

  def put (self, propertylist, valuelist):
    i = 0
    for property in propertylist:
      self._putValue (property, valuelist [i])
      i += 1

  # ---------------------------------------------------------------------------
  # Delete this instance
  # ---------------------------------------------------------------------------

  def delete (self):
    self._record.delete ()

  # ---------------------------------------------------------------------------
  # Call a procedure
  # ---------------------------------------------------------------------------

  def call (self, procedurename, params):
    # TODO: use class repository to fetch method definitions

    # for now: just implement a basic test method
    if procedurename=="show":
      print "Procedure 'show' was called. This record contains the following "+\
            " values:"
      for i in self._record._fields.keys():
        print "%s : %s " % (i,self._record._fields[i])
      print
    else:
      print "The procedure %s is not implemented." % procedurename

    # return value of '1' to make XMLRPC happy
    return 1
