# GNU Enterprise Application Server - Session Object
#
# Copyright 2001-2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: geasSession.py,v 1.40 2003/10/19 16:45:16 reinhard Exp $

import geasList
import classrep
import whrandom

# =============================================================================
# Helper functions
# =============================================================================

# -----------------------------------------------------------------------------
# Generate a new object_id
# -----------------------------------------------------------------------------

def new_object_id ():
  # FIXME: need a better algorithm here
  result = ""
  for i in range (0, 32):
    result = result + str (int (whrandom.random () * 10))
  return result

# =============================================================================
# Session class
# =============================================================================

class geasSession:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self, connections, authAdapter):
    self.loggedIn = 0
    self.connections = connections
    try:
      self.database = gConfig('database')
    except:
      self.database = "gnue"
    self._user = ""
    self._lists = {}
    self._listcount = 0
    self._authAdapter = authAdapter

    # TODO: gnue-common should have a concept of sessions. Until that is done,
    # we have to keep a list of geasList objects that need to be committed.
    self._dirtyLists = []

  # ---------------------------------------------------------------------------
  # Log into the application server
  # ---------------------------------------------------------------------------

  def login (self, user, password):
    # This username/password is for the Application Server, not for the
    # database.
    self._user = user
    self.loggedIn = self._authAdapter.authenticate(self, user,
                                                   {'password':password} )
    return self.loggedIn

  # ---------------------------------------------------------------------------
  # Logout from the application server
  # ---------------------------------------------------------------------------

  def logout (self):
    # should the authAdapter be contacted?
    self.loggedIn = 0

  # ---------------------------------------------------------------------------
  # Create a new list of business objects of a given class
  # ---------------------------------------------------------------------------

  def _createList (self, classname, propertylist, sortorder):

    # check if user has access rights for this class
    if not self._authAdapter.hasAccess (self, self._user, classname):
      raise Exception, "Access to class '%s' denied" % classname

    classdef = classrep.classes [classname]

    # create new List
    return geasList.geasList (self, classdef, propertylist, sortorder)

  # ---------------------------------------------------------------------------
  # Commit the active transaction
  # ---------------------------------------------------------------------------

  def commit (self):
    # TODO: gnue-common should support the concept of a session and commits
    # on session level
    for l in self._dirtyLists:
      if hasattr (l, "_datasource"):
        l._resultset.post ()
        l._datasource.commit ()

  # ---------------------------------------------------------------------------
  # Rollback the active transaction
  # ---------------------------------------------------------------------------

  def rollback (self):
    # TODO: gnue-common should support the concept of a session and rollbacks
    # on session level
    for l in self._dirtyLists:
      if hasattr (l, "_datasource"):
        l._datasource.rollback ()

  # ---------------------------------------------------------------------------
  # Create a new list of business objects of a given class
  # ---------------------------------------------------------------------------

  def request (self, classname, conditions, sortorder, propertylist):
    # FIXME: this list needn't be considered by commit and rollback
    list = self._createList (classname, ["gnue_id"] + propertylist, sortorder)
    list.populate (conditions)
    self._listcount += 1
    self._lists [self._listcount] = list
    list_id = self._listcount
    return list_id;

  # ---------------------------------------------------------------------------
  # Check list id and raise exception if invalid, return list otherwise
  # ---------------------------------------------------------------------------

  def _getList (self, list_id):
    if self._lists.has_key (list_id):
      return self._lists [list_id]
    else:
      raise Exception, "Can't find a list with ID '%s'" % list_id

  # ---------------------------------------------------------------------------
  # Count the number of objects in the list
  # ---------------------------------------------------------------------------

  def count (self, list_id):
    list = self._getList (list_id)
    return list.count ();

  # ---------------------------------------------------------------------------
  # Fetch data from the database backend
  # ---------------------------------------------------------------------------

  def fetch (self, list_id, start, count):
    list = self._getList (list_id)
    return list.fetch (start, count)

  # ---------------------------------------------------------------------------
  # Load data from the database backend
  # ---------------------------------------------------------------------------

  def load (self, classname, obj_id_list, propertylist):
    list = self._createList (classname, ["gnue_id"] + propertylist, [])
    result = []
    for object_id in obj_id_list:
      if object_id == "":
        classdef = classrep.classes [classname]
        result.append ([classdef.properties[p].fullType for p in propertylist])
      else:
        instance = list.find (object_id)
        result.append (instance.get (propertylist))
    return result

  # ---------------------------------------------------------------------------
  # Store data in the database backend
  # ---------------------------------------------------------------------------

  def store (self, classname, obj_id_list, propertylist, data):
    result = []
    i = 0
    for object_id in obj_id_list:
      # TODO: gnue-common should provide a method for updating a record without
      # reading it first. Until that is done, we have to create a temporary
      # list for every record we update and keep it in our list of pending
      # commits.
      list = self._createList (classname, ["gnue_id"] + propertylist, [])
      if object_id:
        instance = list.find (object_id)
        result.append (object_id)
      else:
        instance = list.newInstance ()
        newid = new_object_id ()
        instance.put (["gnue_id"], [newid])
        result.append (newid)
      instance.put (propertylist, data [i])
      self._dirtyLists.append (list)
      i += 1
    return result

  # ---------------------------------------------------------------------------
  # Delete business objects
  # ---------------------------------------------------------------------------

  def delete (self, classname, obj_id_list):
    for object_id in obj_id_list:
      # TODO: gnue-common should provide a method to delete a record without
      # reading it first. Until that is done, we have to create a temporary
      # list for every record we update and keep it in our list of pending
      # commits.
      list = self._createList (classname, ["gnue_id"], [])
      instance = list.find (object_id)
      instance.delete ()
      self._dirtyLists.append (list)

  # ---------------------------------------------------------------------------
  # Call a procedure of business objects
  # ---------------------------------------------------------------------------

  def call (self, classname, obj_id_list, procedurename, parameters):
    list = self._createList (classname, ["gnue_id"], [])
    result = []
    for object_id in obj_id_list:
      instance = list.find (object_id)
      result.append (instance.call (procedurename, parameters))
    return result
