# GNU Enterprise Application Server - Language interface: Object
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Object.py,v 1.10 2003/10/21 07:25:44 reinhard Exp $

import sys
import types
import mx.DateTime
import mx.DateTime.ISO

def newObject (server, sessionId, classname, objData=None):
  return Object (server, sessionId, classname, objData)

# ===========================================================================
# Helper methods
# ===========================================================================

# ---------------------------------------------------------------------------
# Convert RPC transportable datatype into native Python object
# ---------------------------------------------------------------------------

def rpc_to_native (value, type):

  # Empty strings indicate None
  if value == '':
    return None

  # String: convert to unicode
  elif type [:7] == 'string(':
    return unicode (value, 'utf-8')

  # Date: convert to mx.DateTime object
  elif type == 'date':
    return mx.DateTime.ISO.ParseDate (value)

  # Time: convert to mx.DateTime object
  elif type == 'time':
    return mx.DateTime.ISO.ParseTime (value)

  # DateTime: convert to mx.DateTime object
  elif type == 'datetime':
    return mx.DateTime.ISO.ParseDateTime (value)

  # All others (id, number, boolean, reference): no need to convert
  else:
    return value

# ---------------------------------------------------------------------------
# Convert native Python object into RPC transportable datatype
# ---------------------------------------------------------------------------

def native_to_rpc (value):

  if value is None:
    return ''
  elif isinstance (value, types.UnicodeType):
    return value.encode ('utf-8')
  else:
    return str (value)

# ===========================================================================
# Implements a single business object
# ===========================================================================
class Object:

  # -------------------------------------------------------------------------
  # create a new instance: collection is the owning ObjectList and objData
  # is a dictionary as returned by fetch()
  # -------------------------------------------------------------------------
  def __init__ (self, server, sessionId, classname, objData=None):
    # we won't trigger __setattr__ here ...
    self.__dict__['_server']     = server
    self.__dict__['_session_id'] = sessionId
    self.__dict__['_class']      = classname
    self.__dict__['_cache']      = {}
    self.__dict__['objectId']    = None

    # if a dictionary with instance data is given, populate the cache
    if objData:
      self.objectId = objData ['gnue_id']

      # transfer all data from objData to the initial cache
      for key in objData.keys ():
        self._cache [key] = objData [key]

  # -------------------------------------------------------------------------
  # Return an attribute's value; if not available in cache load from server
  # -------------------------------------------------------------------------
  def __getattr__ (self, attr):
    if not self._cache.has_key (attr):
      if self._attrIsValid (attr):
	res = self._server.load (self._session_id, self._class, 
	                         [self.objectId, ''], [attr])
	self._cache [attr] = rpc_to_native (res [0] [0], res [1] [0])
      else:
        raise AttributeError, attr

    return self._cache [attr]


  # -------------------------------------------------------------------------
  # Set an attribute's value
  # -------------------------------------------------------------------------
  def __setattr__ (self, attr, value):
    if self.__dict__.has_key (attr):
      self.__dict__ [attr] = value
    else:
      if not self._attrIsValid (attr):
	raise AttributeError, attr

      self._cache [attr] = value

  # -------------------------------------------------------------------------
  # Check if 'attr' is a valid attribute of this class
  # -------------------------------------------------------------------------
  def _attrIsValid (self, attr):
    # TODO: ask the class repository for the attribute
    # meanwhile we suppose all attributes are valid 
    return 1



  # -------------------------------------------------------------------------
  # Store this instance 
  # -------------------------------------------------------------------------
  def store (self):
    data = []
    prop = []

    # make sure the property-list and the data-list have the same order
    for key in self._cache.keys ():
      if key == 'gnue_id': 
	continue
      prop.append (key)
      data.append (native_to_rpc (self._cache[key]))

    res = self._server.store (self._session_id, self._class, [self.objectId],
                              prop, [data])
    self.objectId = res [0]

  # -------------------------------------------------------------------------
  # Delete instance and clear object-id
  # -------------------------------------------------------------------------
  def delete (self):
    self._server.delete (self._session_id, self._class, [self.objectId])
    self.objectId = None
